"""Provides handful of constants used in record reports/searches"""

# Filter
EQ = "equals"
NOT_EQ = "doesNotEqual"
CONTAINS = "contains"
EXCLUDES = "excludes"
GT = "greaterThan"
LT = "lessThan"
LTE = "lessThanOrEqual"
GTE = "greaterThanOrEqual"

# Aggregate
AVG = "average"
COUNT = "count"
SUM = "sum"
MIN = "min"
MAX = "max"

# GroupBy
GB = "groupBy"
HOUR = "groupByHour"
DAY = "groupByDay"
WEEK = "groupByWeek"
MONTH = "groupByMonth"
QUARTER = "groupByQuarter"
YEAR = "groupByYear"

# Sorts
ASC = "ascending"
DESC = "descending"
